

#include "CWCsbReader.h"
#include "tinyxml2/tinyxml2.h"
#include "flatbuffers/flatbuffers.h"
#include "cocostudio/CSParseBinary_generated.h"
#include "CWCsbOptions_generated.h"
#include "editor-support/cocostudio/CocoLoader.h"
#include "platform/CCFileUtils.h"
#include "CWCsb.h"
USING_NS_CC;
using namespace ui;
using namespace flatbuffers;

static const char* P_FileNameData = "fileNameData";

static CWCsbReader* instanceCWCsbReader = nullptr;

IMPLEMENT_CLASS_NODE_READER_INFO(CWCsbReader)

CWCsbReader::CWCsbReader()
{
	
}

CWCsbReader::~CWCsbReader()
{

}

CWCsbReader* CWCsbReader::getInstance()
{
	if (!instanceCWCsbReader)
	{
		instanceCWCsbReader = new (std::nothrow) CWCsbReader();
	}
	return instanceCWCsbReader;
}

void CWCsbReader::destroyInstance()
{
	CC_SAFE_DELETE(instanceCWCsbReader);
}
/*
void CWCsbReader::setPropsFromBinary(cocos2d::ui::Widget *widget, CocoLoader *cocoLoader, stExpCocoNode *cocoNode)
{
	this->beginSetBasicProperties(widget);

	CWCsb* nodeCsb = static_cast<CWCsb*>(widget);

	stExpCocoNode *stChildArray = cocoNode->GetChildArray(cocoLoader);

	for (int i = 0; i < cocoNode->GetChildNum(); ++i) {
		std::string key = stChildArray[i].GetName(cocoLoader);
		std::string value = stChildArray[i].GetValue(cocoLoader);
		//read all basic properties of widget
		CC_BASIC_PROPERTY_BINARY_READER
		//read all color related properties of widget
		CC_COLOR_PROPERTY_BINARY_READER

		else if (key == P_FileNameData){
			stExpCocoNode *backGroundChildren = stChildArray[i].GetChildArray(cocoLoader);
			std::string backgroundValue = this->getResourcePath(cocoLoader, &stChildArray[i], Widget::TextureResType::LOCAL);
			nodeCsb->setCsbFile(backgroundValue);
		}

	} //end of for loop
	this->endSetBasicProperties(widget);
}

void CWCsbReader::setPropsFromJsonDictionary(Widget *widget, const rapidjson::Value &options)
{
	WidgetReader::setPropsFromJsonDictionary(widget, options);

	std::string jsonPath = GUIReader::getInstance()->getFilePath();

	CWCsb* nodeCsb = static_cast<CWCsb*>(widget);

	const rapidjson::Value& cmftDic = DICTOOL->getSubDictionary_json(options, P_FileNameData);
	
	std::string tp_c = jsonPath;
	const char* cmfPath = DICTOOL->getStringValue_json(cmftDic, P_Path);
	const char* cmf_tp = tp_c.append(cmfPath).c_str();
	nodeCsb->setCsbFile(cmf_tp);
				 
	WidgetReader::setColorPropsFromJsonDictionary(widget, options);
}
*/

Offset<Table> CWCsbReader::createOptionsWithFlatBuffers(const tinyxml2::XMLElement *objectData,
	flatbuffers::FlatBufferBuilder *builder)
{
	auto temp = WidgetReader::getInstance()->createOptionsWithFlatBuffers(objectData, builder);
	auto widgetOptions = *(Offset<WidgetOptions>*)(&temp);

	std::string path = "";

	const tinyxml2::XMLElement* child = objectData->FirstChildElement();
	while (child)
	{
		std::string name = child->Name();
		
		if (name == "CsbResourceFile_CNB")
		{
			const tinyxml2::XMLAttribute* attribute = child->FirstAttribute();
			while (attribute)
			{
				name = attribute->Name();
				std::string value = attribute->Value();
				if (name == "Path")
				{
					path = value;
				}
				attribute = attribute->Next();
			}
		}
		child = child->NextSiblingElement();
	}

	auto options = flatbuffers::CreateCWCsbOptions(*builder,
		widgetOptions,
		CreateResourceData(*builder, builder->CreateString(path), builder->CreateString(""), 0));

	return *(Offset<Table>*)(&options);
}

void CWCsbReader::setPropsWithFlatBuffers(cocos2d::Node *node,const flatbuffers::Table* singleNodeOptions)
{
	CWCsb* nodeCWCsb = static_cast<CWCsb*>(node);
	auto options = (flatbuffers::CWCsbOptions*)singleNodeOptions;

	auto cmftDic = options->fileNameData();
	bool fileExist = false;

	std::string path = cmftDic->path()->c_str();
	if (path.length() > 0)
	{
		path = path.substr(0, path.length() - 3).append("csb");
	}
	if (FileUtils::getInstance()->isFileExist(path))
	{
		fileExist = true;
	}

	if (fileExist)
	{
		nodeCWCsb->setCsbFile(path);
	}

	auto widgetReader = WidgetReader::getInstance();
	widgetReader->setPropsWithFlatBuffers(node, (Table*)options->widgetOptions());
}

Node* CWCsbReader::createNodeWithFlatBuffers(const flatbuffers::Table *nodeOptions)
{
	cocos2d::ui::CWCsb* node = cocos2d::ui::CWCsb::create();

	// self
	setPropsWithFlatBuffers(node, (Table*)nodeOptions);

	return node;
}


